
import openpyxl
import os
import pandas as pd
import json
import collections
import argparse

from .Book import Book
from .ProgressBar import printProgressBar
from .BookCollection import BookCollection


# LINE ARGUMENTS ------------------------------------------------------------------------

def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("-p", "--Path", help="Origin path", required=True)
    parser.add_argument("-l", "--LogFilePath", help="Log File path", required=False)
    parser.add_argument("-e", "--Extensions", nargs='+', help="Supported Extensions", required=False)

    parser.add_argument("-t", "--NotionToken", help="Notion Token", required=False)
    parser.add_argument("-u", "--NotionDbUrl", help="Notion DB URL", required=False)
    parser.add_argument("-f", "--ForceUpload", help="Force upload to Notion", required=False, action='store_true')
    parser.add_argument("-o", "--OutputFolder", help="Folder where .csv and .xlsx files will be created", required=False)

    args = parser.parse_args()

    if args.Path: path = args.Path
    if args.Extensions:
        supported_extensions = args.Extensions
    else:
        supported_extensions = ['.epub', '.pdf', '.paper']
    if args.NotionToken: notionToken = args.NotionToken
    if args.NotionDbUrl: notionDbUrl = args.NotionDbUrl
    forceUpload = False
    if args.ForceUpload: forceUpload = True
    if args.OutputFolder:
        outputFolder = args.OutputFolder
    else:
        outputFolder = './l2n-output'
    if args.LogFilePath:
        logFilePath = args.LogFilePath
    else:
        logFilePath = outputFolder + '/log.json'


    # ---------------------------------------------------------------------------------------

    print('\nlibrary2notion created by Javier Helguera (github.com/helguera) © 2022 MIT License\n')

    wb = openpyxl.Workbook()
    ws1 = wb.active
    ws1.title = "books"

    ws1.cell(row=1, column=1).value = 'File Name'
    ws1.cell(row=1, column=2).value = 'Title'
    ws1.cell(row=1, column=3).value = 'Priority'
    ws1.cell(row=1, column=4).value = 'Tags'
    ws1.cell(row=1, column=5).value = 'Status'
    ws1.cell(row=1, column=6).value = 'Author'
    ws1.cell(row=1, column=7).value = 'Publisher'
    ws1.cell(row=1, column=8).value = 'Comments'
    ws1.cell(row=1, column=9).value = 'Format'
    ws1.cell(row=1, column=10).value = 'ISBN'


    #####################################################################################################################

    files_path = [os.path.abspath(os.path.join(path, x)) for x in os.listdir(path)]
    files = []
    temp_files = []

    for r, d, f in os.walk(path):
        for file in f:
            if filter(lambda element: file in element, supported_extensions):
                files.append(os.path.join(r, file))
    temp_files = files.copy()
    temp_files = filter(lambda x: '._' not in x, files)
    files = filter(lambda x: '._' not in x, files)

    bookCollection = BookCollection()

    total_files = 0
    for index, my_file in enumerate(temp_files, start=1):
        total_files += 1

    print('\nTOTAL FILES: ' + str(total_files) + '\n')
    printProgressBar(0, total_files, prefix = 'Detecting files:       ', suffix = 'Complete', length = 100)

    for index, my_file in enumerate(files, start=1):

        if os.path.splitext(my_file)[1] in supported_extensions:
            book = Book(os.path.splitext(my_file)[0], [os.path.splitext(my_file)[1].replace('.','').upper()], os.path.splitext(my_file)[1])
            bookCollection.insert(book)

        printProgressBar(index, total_files, prefix = 'Detecting files:       ', suffix = 'Complete', length = 100)

    # JSON PROCESSING
    deleted = []
    if ('logFilePath' in locals() and os.path.exists(logFilePath) and not forceUpload):
        with open(logFilePath, 'r') as openfile:
            # Reading from json file
            json_object = json.load(openfile)
            printProgressBar(0, len(json_object)-1, prefix = 'Analazing log file:    ', suffix = 'Complete', length = 100)
            for index, value in enumerate(json_object):
                bookInCollection = bookCollection.findBook(value['filename'])
                if bookInCollection is not None:
                    if collections.Counter(bookInCollection.formats) == collections.Counter(value['formats']):
                        bookInCollection.action = "do nothing"
                    else:
                        json_object[index]['formats']= bookInCollection.formats
                        bookInCollection.action = "update"
                else:
                    json_object[index]['action']= 'DELETED'
                    deleted.append(json_object[index])
                printProgressBar(index, len(json_object)-1, prefix = 'Analazing log file:    ', suffix = 'Complete', length = 100)

        deleted_json_object = json.dumps(deleted, indent=2)
        with open(os.path.join(outputFolder, 'deleted.json'), "w") as outfile:
            outfile.write(deleted_json_object)

    json_object = json.dumps(bookCollection.createJson(), indent=2)
    if not os.path.exists(outputFolder):
        os.makedirs(outputFolder)
    with open(logFilePath, "w") as outfile:
        outfile.write(json_object)

    # bookCollection.printAll()
    bookCollection.extractMetadataFromBooks(forceUpload)
    bookCollection.copyAllToExcel(ws1)
    if not os.path.exists(outputFolder):
        os.mkdir(outputFolder)
    wb.save(filename=os.path.join(outputFolder, 'books.xlsx'))
    df = pd.read_excel(os.path.join(outputFolder, 'books.xlsx'), 'books', index_col=None)
    df.to_csv (os.path.join(outputFolder, 'books.csv'), index = None, header=True, encoding='utf-8-sig')
    printProgressBar(0, 0, prefix = 'Converting to .csv:    ', suffix = 'Complete', length = 100)

    if not bookCollection.isEmpty() or forceUpload:
        if 'notionToken' in locals() and 'notionDbUrl' in locals():
            print('\n')
            command = 'csv2notion --token ' + notionToken + ' --url ' + notionDbUrl + ' --merge ' + os.path.join(outputFolder, 'books.csv') + ' --merge-only-column "File Name" --merge-only-column "Title" --merge-only-column "Author" --merge-only-column "Publisher" --merge-only-column "Format" --merge-only-column "Tags" --merge-only-column "ISBN" --column-types "text, select, multi_select, select, text, text, text, multi_select, text" --add-missing-columns --verbose'
            os.system(command)
        else:
            print('\n\nCSV file has been created but no Notion data was provided')
    else:
        print('\n\nEverything up to date. Nothing to upload to Notion.')

    print('\nDONE.\n')



from .ProgressBar import printProgressBar

class BookCollection:
    def __init__(self):
        self.bookCollection = []

    def insert(self, book): 
        if not self.existsByFileName(book.fileName):
            self.bookCollection.append(book)
        else:
            duplicated = next(x for x in self.bookCollection if x.fileName == book.fileName)
            duplicated.formats.append(book.temp_format.upper().replace('.', ''))


    def existsByFileName(self, fileName):
        if len([x for x in self.bookCollection if x.fileName == fileName]) != 0:
            return True
        return False

    def copyAllToExcel(self, ws):
        rowNum = 2
        length = len(self.bookCollection)
        printProgressBar(0, length-1, prefix = 'Generating .xlsx file:', suffix = 'Complete', length = 100)
        for index, book in enumerate(self.bookCollection):
            printProgressBar(index, length-1, prefix = 'Generating .xlsx file: ', suffix = 'Complete', length = 100)
            if book.action in ['update', 'upload']:
                book.copyToExcel(rowNum, ws)
                rowNum += 1

    def printAll(self):
        for book in self.bookCollection:
            print(book.fileName + ' - ' + str(book.formats))

    def createJson(self):
        mylist = []
        for book in self.bookCollection:
            mylist.append({'filename': book.fileName, 'action': book.action, 'formats': book.formats})
        return mylist

    def findBook(self, fileName,):
        listBooks = [x for x in self.bookCollection if x.fileName == fileName]
        if len(listBooks) != 0:
            return listBooks[0]
        return None

    def removeBook(self, book):
        self.bookCollection.remove(book)

    def isEmpty(self):
        for book in self.bookCollection:
            if book.action in ['upload', 'update']:
                return False
        return True

    def extractMetadataFromBooks(self, forceUpload):
        length = len(self.bookCollection)
        printProgressBar(0, length, prefix = 'Extracting metadata:   ', suffix = 'Complete', length = 100)
        for index, book in enumerate(self.bookCollection):
            if (book.action in ['update', 'upload']) or forceUpload:
                book.extractMetadata()
            printProgressBar(index+1, length, prefix = 'Extracting metadata:   ', suffix = 'Complete', length = 100)




def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    if iteration == 0 and total == 0:
        bar = fill*100
        print(f'\r{prefix} |{bar}| 100.0% {suffix}', end = printEnd)
        return
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print(f'\r{prefix} |{bar}| {percent}% {suffix}', end = printEnd)
    # Print New Line on Complete
    if iteration == total: 
        print()


import openpyxl
import epub_meta
from PyPDF2 import PdfReader
import json


class Book:
    def __init__(self, fileName, formats, temp_format):
        self.fileName = fileName
        self.title = ''
        self.priority = ''
        tags = (fileName + temp_format).split('/')[:-1]
        self.tags = tags
        self.status = ''
        self.author = ''
        self.publisher = ''
        self.comments = ''
        self.formats = formats
        self.temp_format = temp_format
        self.isbn = ''
        self.action = 'upload'

    def copyToExcel(self, rowNum, ws):
        ws.cell(row=rowNum, column=1).value = self.fileName
        ws.cell(row=rowNum, column=2).value = self.title
        ws.cell(row=rowNum, column=3).value = self.priority
        ws.cell(row=rowNum, column=4).value = self.tags
        ws.cell(row=rowNum, column=5).value = self.status
        ws.cell(row=rowNum, column=6).value = self.author
        ws.cell(row=rowNum, column=7).value = self.publisher
        ws.cell(row=rowNum, column=8).value = self.comments
        ws.cell(row=rowNum, column=9).value = ', '.join(self.formats)
        ws.cell(row=rowNum, column=10).value = self.isbn

    def extractMetadata(self):
        for format in self.formats:
            filePath = self.fileName + '.' + format.lower()

            if 'EPUB' in format.upper():
                data = epub_meta.get_epub_metadata(filePath, read_cover_image=True, read_toc=True)

                # TITLE
                self.title = data.title

                # AUTHOR
                self.author = ' '.join(data.authors)

                # PUBLISHER
                if data.publisher:
                    self.publisher = data.publisher.replace(',','')

                # ISBN
                self.isbn = ' '.join(data.identifiers)

            elif 'PDF' in format.upper():
                try:
                    reader = PdfReader(filePath)
                    pdf_info = reader.metadata

                    # TITLE
                    if pdf_info.title:
                        self.title = pdf_info.title
                    else:
                         self.title = filePath.split('/')[len(filePath.split('/')) - 1].split('.')[0]

                    # AUTHOR
                    author = ''
                    if pdf_info.author is not None:
                        self.author = pdf_info.author

                except Exception as e:
                    if str(e) != 'EOF marker not found':
                        self.status = 'BROKEN!'

            elif 'PAPER' in format.upper():
                f = open(filePath)
                data = json.load(f)
                try:
                    self.title = data['Title']
                    self.author = data['Author']
                    self.publisher = data['Publisher']
                    self.isbn = data['ISBN']
                except:
                    pass


        if self.tags[len(self.tags) - 1] == self.title:
            self.tags = self.tags[:-1]
        self.tags = ', '.join(self.tags[1:])


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['library2notion']

package_data = \
{'': ['*']}

install_requires = \
['argparse>=1.4.0,<2.0.0',
 'csv2notion>=0.3.7,<0.4.0',
 'epub-meta>=0.0.7,<0.0.8',
 'openpyxl>=3.0.10,<4.0.0',
 'pandas>=1.5.2,<2.0.0',
 'pypdf2>=2.11.2,<3.0.0']

entry_points = \
{'console_scripts': ['library2notion = library2notion.library2notion:main']}

setup_kwargs = {
    'name': 'library2notion',
    'version': '0.2.0',
    'description': '',
    'long_description': '# library2notion\n\n[![PyPI version](https://img.shields.io/pypi/v/library2notion?label=version)](https://pypi.python.org/pypi/library2notion)\n[![Python Version](https://img.shields.io/pypi/pyversions/library2notion.svg)](https://pypi.org/project/library2notion/)\n\nA way to upload all your digital library in format .PDF or .EPUB to [Notion.so](https://notion.so). This tools will upload to a database in Notion relevant information from a digital library in order to make it easier to find books, add comments, filter by type or category and keep track of read ones.\n\nThis tool makes use of **csv2notion**, a tool created by [vzhd1701](https://github.com/vzhd1701/csv2notion).\n\n## Installation\n\n### Using portable binary\n\n[**Download the latest binary release**](https://github.com/helguera/library2notion/releases/latest).\n\n### With [Homebrew](https://brew.sh/) (Recommended for macOS)\n\n```bash\n$ brew install helguera/tap/library2notion\n```\n\n### With [PIPX](https://github.com/pypa/pipx) (Recommended for Linux & Windows)\n\n```shell\n$ pipx install library2notion\n```\n\n### With PIP\n\n```bash\n$ pip install --user library2notion\n```\n\n**Python 3.7 or later required.**\n\n### From source\n\nThis project uses [poetry](https://python-poetry.org/) for dependency management and packaging. You will have to install it first. See [poetry official documentation](https://python-poetry.org/docs/) for instructions.\n\n```shell\n$ git clone https://github.com/helguera/library2notion.git\n$ cd library2notion/\n$ poetry install\n$ poetry run library2notion\n```\n\n## Usage\n\n```plain\n$ library2notion --help\nusage: library2notion [-h] -p PATH [-l LOGFILEPATH] [-e EXTENSIONS [EXTENSIONS ...]] [-t NOTIONTOKEN] [-u NOTIONDBURL] [-f] [-o OUTPUTFOLDER]\n\nImport/Merge your digital library collection in .PFD or .EPUB format to Notion.\n\ngeneral options:\n  -p, --Path PATH                    Path where to start looking for books. It will also check all subfolders\n  -l, --LogFilePath LOGFILEPATH      Location of the log file if exists. If not, a new one will be created in the specified location\n  -e, --Extensions EXTENSIONS        List of extensions to be taken into account. At this moment .PDF, .EPUB and .PAPER are supported.\n  -t, --NotionToken NOTIONTOKEN      Notion token, stored in token_v2 cookie for notion.so\n  -u, --NotionDbUrl NOTIONDBURL      Notion DB URL\n  -f, --ForceUpload FORCEUPLOAD      Force upload to Notion (if token and db url are provided) ignoring the log file. If no Notion data is provided it will generate the .csv file.\n  -o, --OutputFolder OUTPUTFOLDER    Folder to store .csv, .xlsx, and deleted.json files. If not provided it will use \'./\'\n  -h, --help                         show this help message and exit\n```\n\n### Input\n\nYou must pass a starting path for the application to start searching for books with the `--Path` option. Also, a log file is needed if you have already executed the tool before. It can be passed with the `--LogFilePath` option. This log file will contain the status of previous execution so books that have already been scanned and upload won\'t be affected. If this log file is not provided, a new one will be created.\n\nOptionally you can specify what book extensions you want the application to analyze with the option `--Extensions`. At this moment .PDF and .EPUB are supported.\n\nIf you want the application to upload the data to Notion, you must provide a URL to an existing Notion database with the `--NotionDbUrl` option; The URL must link to a database view, not a page.\n\nThe tool also requires you to provide a `token_v2` cookie for the Notion website through `--NotionToken` option.\n\n**Important notice**. `token_v2` cookie provides complete access to your Notion account. Handle it with caution.\n\n### Metadata\n\nThe tool will extract the following data to upload to Notion:\n\n- **File Name**: is the full path to the file. It is used as primary key of the table in Notion, so it can\'t be duplicated.\n- **Title**: title of the book.\n- **Tags**: the categories of the book. They are generated automatically from the path. For example, if the path is `./Tech Books/Programming/Python/mybook.pdf`, the tags will be `Tech Books`, `Programming`, `Python`.\n- **Author**: the author or authors of the book.\n- **Publisher**: the publisher of the book.\n- **Formats**: the available formats of the book. A book available in multiple formats will only appear once in the database.\n- **ISBN**: the ISBN.\n\n### Columns\n\nThe tool will create a column per metadata extracted. Also, the following ones, which will be empty because they are only intended to be used in Notion, will be created:\n\n- **Priority**: allows to select a priority and filter by that property in Notion.\n- **Status**: the status of the book (reading, not started, on hold...)\n- **Comments**: if we need to add some comments to the book\n\n### Paper Books\n\nWith update 0.2.0, the tool supports physical books. You just have to create a `.paper` file in a folder per physical book that you want to add with the following content:\n\n```json\n{\n    "Title": "",\n    "Author": "",\n    "Publisher": "",\n    "ISBN": ""\n}\n```\n\n### Log File\n\nThe log file is a json file that contains info about the books that have already been uploaded to Notion. It will be automatically generated the first time the application is used. In next executions, if one or more files has been added to the path, the log file must be provided so the tool knows that only those new books have to be scanned and uploaded.\n\n### Deleted Books\n\nIf a book is deleted from the path, the tool will detect it the next time it is executed (only, of course, if a log file is provided). This will generate a `deleted.json` file in the ouput folder with the deleted books. But, **really important**, the book will not be deleted from Notion. That has to be done manually.\n\n## csv2notion\n\nThis tool analizes books and creates a .csv file with the results. The task of uploading the data to Notion is performed by the tool [csv2notion](https://github.com/vzhd1701/csv2notion).\n\n## Getting help\n\nIf you found a bug or have a feature request, please [open a new issue](https://github.com/helguera/library2notion/issues/new/choose).\n\nIf you have a question about the program or have difficulty using it, you are welcome to [the discussions page](https://github.com/helguera/library2notion/discussions). You can also mail me directly at [javier@javierhelguera.com](mailto:javier@javierhelguera.com), I\'m always happy to help.',
    'author': 'helguera',
    'author_email': 'javier@javierhelguera.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
